@@ -22,6 +22,7 @@
 #define __UP_DAEMON_H__
 
 #include <dbus/up-daemon-generated.h>
+
 #include "up-types.h"
 #include "up-device-list.h"
 
@@ -86,6 +87,10 @@ const gchar	*up_daemon_get_charge_icon	(UpDaemon		*daemon,
 						 gdouble		 percentage,
 						 UpDeviceLevel		 battery_level,
 						 gboolean		 charging);
+const gchar	*up_daemon_get_state_dir_env_override (UpDaemon *daemon);
+gboolean	 up_daemon_polkit_is_allowed	(UpDaemon		*daemon,
+						 const gchar		*action_id,
+						 GDBusMethodInvocation	*invocation);
 
 void             up_daemon_pause_poll           (UpDaemon               *daemon);
 void             up_daemon_resume_poll          (UpDaemon               *daemon);
